package edu.rice.rubis.beans;

import javax.ejb.CreateException;
import javax.ejb.EJBLocalHome;
import javax.ejb.FinderException;
import javax.ejb.RemoveException;
import java.rmi.RemoteException;
import java.util.Collection;

/** This is the LocalHome interface of the Category Bean */

public interface CategoryLocalHome extends EJBLocalHome {

  /**
   * This method is used to create a new Category Bean. Note that the category
   * id is automatically generated by the database (AUTO_INCREMENT) on the
   * primary key.
   *
   * @param name Category name
   *
   * @return pk primary key set to null
   */
  public CategoryLocal create(String name) throws CreateException, RemoteException, RemoveException;


  /**
   * This method is used to retrieve a Category Bean from its primary key,
   * that is to say its id.
   *
   * @param id Category id (primary key)
   *
   * @return the Category if found else null
   */
  public CategoryLocal findByPrimaryKey(CategoryPK id) throws FinderException, RemoteException;


  /**
   * This method is used to retrieve a Category Bean from its name.
   *
   * @param categoryName Category name
   *
   * @return the Category if found else null
   */
  public CategoryLocal findByName(String categoryName) throws FinderException, RemoteException;


  /**
   * This method is used to retrieve all categories from the database!
   *
   * @return List of all categories (eventually empty)
   */
  public Collection findAllCategories() throws RemoteException, FinderException;
}
